'use client'
import React, { memo, useState } from 'react'
import { ChevronDownIcon, ChevronUpIcon } from "@chakra-ui/icons";
import { Avatar, Box, Button, Fade, Flex, Text } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { getStorage, removeStorage, setStorage } from '@/utils/navigation';
import { createConsumerRequest } from '@/app/lib/consumers/action';
import { editAttempt, storeAttempt } from '@/app/lib/attempt/action';
import useShowToast from '@/app/hooks/useShowToast';
import { onCompleteImage } from '@/utils/common';
import { ConsumerData } from '../page';
import { BsEye } from 'react-icons/bs';
import moment from 'moment';
import { ConsumerInterface } from '../type/consumer';

export interface ConsumerCardProps {
  consumer: ConsumerInterface;
  isOpen: boolean;
  onToggle: () => void;
  fetch: () => void;
}

const Card: React.FC<ConsumerCardProps> = ({ consumer, isOpen, onToggle, fetch }) => {
  const router = useRouter();
  const showToast = useShowToast();
  const [loading, setLoading] = useState(false)


  const handleAttemptUpdate = async (assignmentId: number, consumerId: number) => {
    try {
      const response = await editAttempt(assignmentId, { consumer_id: consumerId });
      if (response) {
        removeStorage("attempted_update");
        router.push("/officer/assignments");
      }
    } catch (error: any) {
      showToast("Failed to update attempt", error.message, "error");
    }
  };
  const handleCreateAttempt = async () => {
    try {
      const data = await getStorage("attempted_update");

      const response = await storeAttempt({ ...data });
      return response;
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
      return null;
    }
  }

  const handleFilterOfflineData = async () => {
    const storedData = await getStorage('consumers');
    const newData = storedData?.data?.filter((item: { id: number }) => item?.id != consumer?.id);

    setStorage('consumers', { last_sync_at: storedData?.last_sync_at, data: newData || [] });
  }


  const handleCreateConsumer = async () => {
    setLoading(true);
    try {
      const uploadedData = consumer?.files?.length > 0 ? await onCompleteImage(consumer?.files) : '';

      const response = await createConsumerRequest({
        contact_person: consumer?.contact_person ?? '',
        dc_request: consumer?.dc_request ?? '',
        meter_reading: consumer?.meter_reading ?? '',
        meter_no: consumer?.meter_no ?? '',
        meter_un_meter: consumer?.meter_un_meter ?? '',
        mobile_no: consumer?.contact_person ?? '',
        paid_amount: consumer?.paid_amount ?? '',
        remarks: consumer?.remarks ?? '',
        latitude: consumer?.latitude ?? '',
        longitude: consumer?.longitude ?? '',
        created_at: consumer?.created_at ?? '',
        updated_at: consumer?.updated_at ?? '',
        files: uploadedData?.length > 0 ? uploadedData : []
      });
      if (response) {
        removeStorage("attempted");
        const attemptData = await getStorage("attempted_update");
        if (attemptData) await handleAttemptUpdate(attemptData?.id, response?.data?.id);

        handleFilterOfflineData()
        showToast("Success", response?.message, "success");

      }
    } catch (error: any) {
      showToast("Failed to create consumer", error.message, "error");
    } finally {
      setLoading(false);
    }
  }

  const handleUpload = async () => {
    const res = await handleCreateAttempt()
    res && handleCreateConsumer()
  }

  return (
    <Box border="1px" p={3} rounded="20px" mt="30px">
      <Flex gap={2} alignItems="center" w="100%">
        <Flex flexDirection="row" alignItems="center" w="100%" gap={2} cursor="pointer">
          <Avatar name={consumer.contact_person} size="sm" />

          <Text fontSize="lg" fontWeight="bold" textTransform="uppercase" onClick={onToggle}>
            {consumer?.contact_person} | {consumer?.mobile_no}
          </Text>

          <Box
            onClick={() => {
              router.push(`/officer/consumers/${consumer?.id}`);
            }}>
            <BsEye width="20px" height="20px" />
          </Box>

          <Box ms="auto" onClick={onToggle}>
            {isOpen ? <ChevronUpIcon width="20px" height="20px" /> : <ChevronDownIcon width="20px" height="20px" />}
          </Box>
        </Flex>
      </Flex>
      {isOpen && (
        <Fade in>
          <Box py="1" mt={3} px={0} rounded="md">
            <Flex rowGap={1} alignItems="center" flexWrap="wrap" justifyContent="space-between">
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Contact Person: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.contact_person}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Mobile No: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.mobile_no}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Paid Amount: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.paid_amount}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  DC Request: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.dc_request == 1 ? "Yes" : "No"}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Meter/ UnMeter: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.meter_un_meter == 1 ? "Yes" : "No"}
                  </Text>
                </Text>
              </Box>
              {consumer?.meter_un_meter == 1 && (
                <>
                  <Box width="50%">
                    <Text fontSize="sm" fontWeight="bold">
                      Meter Reading: &nbsp;
                      <Text as="span" fontWeight="normal">
                        {consumer?.meter_reading}
                      </Text>
                    </Text>
                  </Box>
                  <Box width="50%">
                    <Text fontSize="sm" fontWeight="bold">
                      Meter No: &nbsp;
                      <Text as="span" fontWeight="normal">
                        {consumer?.meter_no}
                      </Text>
                    </Text>
                  </Box>
                </>
              )}
              {consumer?.meter_un_meter == 0 && (
                <Box width="50%">
                  <Text fontSize="sm" fontWeight="bold">
                    Reason for Un meter: &nbsp;
                    <Text as="span" fontWeight="normal">
                      {consumer?.un_meter_reason}
                    </Text>
                  </Text>
                </Box>
              )}
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Remarks: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.remarks}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Additional Remarks: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.additional_remarks}
                  </Text>
                </Text>
              </Box>

              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Created At: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {moment(consumer?.created_at).fromNow()}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Updated At: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {moment(consumer?.updated_at).fromNow()}
                  </Text>
                </Text>
              </Box>
            </Flex>
          </Box>
        </Fade>
      )}

      {consumer?.offline && (
        <Flex columnGap={1} alignItems="center" mt="6">
          <Button onClick={handleUpload} size="sm" px="3" isDisabled={loading} isLoading={loading}>
            Upload
          </Button>
        </Flex>
      )}
    </Box>
  );
};

export default memo(Card)